import unittest

from leeger.model.league import YearSettings
from leeger.model.league.League import League
from leeger.model.league.Matchup import Matchup
from leeger.model.league.Week import Week
from leeger.model.league.Year import Year
from leeger.util.excel_helper import allTimeTeamsStatSheet
from test.helper.prototypes import getNDefaultOwnersAndTeams


class TestExcelHelper(unittest.TestCase):

    def test_allTimeTeamsStatSheet_happyPath(self):
        owners, teams = getNDefaultOwnersAndTeams(2)

        matchup = Matchup(teamAId=teams[0].id, teamBId=teams[1].id, teamAScore=1, teamBScore=2)
        week = Week(weekNumber=1, matchups=[matchup])
        year = Year(yearNumber=2000, teams=teams, weeks=[week])

        league = League(name="TEST", owners=owners, years=[year])

        allTimeTeamsStatSheet_ = allTimeTeamsStatSheet(league)

        self.assertEquals(33, len(allTimeTeamsStatSheet_))
        for tup in allTimeTeamsStatSheet_:
            self.assertIsInstance(tup, tuple)
        self.assertEquals("Team", allTimeTeamsStatSheet_[0][0])
        self.assertEquals("Owner", allTimeTeamsStatSheet_[1][0])
        self.assertEquals("Year", allTimeTeamsStatSheet_[2][0])
        self.assertEquals("Games Played", allTimeTeamsStatSheet_[3][0])
        self.assertEquals("Wins", allTimeTeamsStatSheet_[4][0])
        self.assertEquals("Losses", allTimeTeamsStatSheet_[5][0])
        self.assertEquals("Ties", allTimeTeamsStatSheet_[6][0])
        self.assertEquals("Win Percentage", allTimeTeamsStatSheet_[7][0])
        self.assertEquals("WAL", allTimeTeamsStatSheet_[8][0])
        self.assertEquals("WAL Per Game", allTimeTeamsStatSheet_[9][0])
        self.assertEquals("AWAL", allTimeTeamsStatSheet_[10][0])
        self.assertEquals("AWAL Per Game", allTimeTeamsStatSheet_[11][0])
        self.assertEquals("Opponent AWAL", allTimeTeamsStatSheet_[12][0])
        self.assertEquals("Opponent AWAL Per Game", allTimeTeamsStatSheet_[13][0])
        self.assertEquals("Smart Wins", allTimeTeamsStatSheet_[14][0])
        self.assertEquals("Smart Wins Per Game", allTimeTeamsStatSheet_[15][0])
        self.assertEquals("Opponent Smart Wins", allTimeTeamsStatSheet_[16][0])
        self.assertEquals("Opponent Smart Wins Per Game", allTimeTeamsStatSheet_[17][0])
        self.assertEquals("Points Scored", allTimeTeamsStatSheet_[18][0])
        self.assertEquals("Points Scored Per Game", allTimeTeamsStatSheet_[19][0])
        self.assertEquals("Opponent Points Scored", allTimeTeamsStatSheet_[20][0])
        self.assertEquals("Opponent Points Scored Per Game", allTimeTeamsStatSheet_[21][0])
        self.assertEquals("Scoring Share", allTimeTeamsStatSheet_[22][0])
        self.assertEquals("Opponent Scoring Share", allTimeTeamsStatSheet_[23][0])
        self.assertEquals("Max Scoring Share", allTimeTeamsStatSheet_[24][0])
        self.assertEquals("Min Scoring Share", allTimeTeamsStatSheet_[25][0])
        self.assertEquals("Max Score", allTimeTeamsStatSheet_[26][0])
        self.assertEquals("Min Score", allTimeTeamsStatSheet_[27][0])
        self.assertEquals("Scoring Standard Deviation", allTimeTeamsStatSheet_[28][0])
        self.assertEquals("Plus/Minus", allTimeTeamsStatSheet_[29][0])
        self.assertEquals("Team Score", allTimeTeamsStatSheet_[30][0])
        self.assertEquals("Team Success", allTimeTeamsStatSheet_[31][0])
        self.assertEquals("Team Luck", allTimeTeamsStatSheet_[32][0])

    def test_allTimeTeamsStatSheet_leagueMedianGamesIsTrueInAnyYearSettings(self):
        owners, teams = getNDefaultOwnersAndTeams(2)

        matchup = Matchup(teamAId=teams[0].id, teamBId=teams[1].id, teamAScore=1, teamBScore=2)
        week = Week(weekNumber=1, matchups=[matchup])
        yearSettings = YearSettings(leagueMedianGames=True)
        year1 = Year(yearNumber=2000, teams=teams, weeks=[week], yearSettings=yearSettings)

        _, teams = getNDefaultOwnersAndTeams(2)
        teams[0].ownerId = owners[0].id
        teams[1].ownerId = owners[1].id

        matchup = Matchup(teamAId=teams[0].id, teamBId=teams[1].id, teamAScore=1, teamBScore=2)
        week = Week(weekNumber=1, matchups=[matchup])
        year2 = Year(yearNumber=2001, teams=teams, weeks=[week])

        league = League(name="TEST", owners=owners, years=[year1, year2])

        allTimeTeamsStatSheet_ = allTimeTeamsStatSheet(league)

        self.assertEquals(36, len(allTimeTeamsStatSheet_))
        for tup in allTimeTeamsStatSheet_:
            self.assertIsInstance(tup, tuple)
        self.assertEquals("Team", allTimeTeamsStatSheet_[0][0])
        self.assertEquals("Owner", allTimeTeamsStatSheet_[1][0])
        self.assertEquals("Year", allTimeTeamsStatSheet_[2][0])
        self.assertEquals("Games Played", allTimeTeamsStatSheet_[3][0])
        self.assertEquals("Total Games", allTimeTeamsStatSheet_[4][0])
        self.assertEquals("Wins", allTimeTeamsStatSheet_[5][0])
        self.assertEquals("Losses", allTimeTeamsStatSheet_[6][0])
        self.assertEquals("League Median Wins", allTimeTeamsStatSheet_[7][0])
        self.assertEquals("Opponent League Median Wins", allTimeTeamsStatSheet_[8][0])
        self.assertEquals("Ties", allTimeTeamsStatSheet_[9][0])
        self.assertEquals("Win Percentage", allTimeTeamsStatSheet_[10][0])
        self.assertEquals("WAL", allTimeTeamsStatSheet_[11][0])
        self.assertEquals("WAL Per Game", allTimeTeamsStatSheet_[12][0])
        self.assertEquals("AWAL", allTimeTeamsStatSheet_[13][0])
        self.assertEquals("AWAL Per Game", allTimeTeamsStatSheet_[14][0])
        self.assertEquals("Opponent AWAL", allTimeTeamsStatSheet_[15][0])
        self.assertEquals("Opponent AWAL Per Game", allTimeTeamsStatSheet_[16][0])
        self.assertEquals("Smart Wins", allTimeTeamsStatSheet_[17][0])
        self.assertEquals("Smart Wins Per Game", allTimeTeamsStatSheet_[18][0])
        self.assertEquals("Opponent Smart Wins", allTimeTeamsStatSheet_[19][0])
        self.assertEquals("Opponent Smart Wins Per Game", allTimeTeamsStatSheet_[20][0])
        self.assertEquals("Points Scored", allTimeTeamsStatSheet_[21][0])
        self.assertEquals("Points Scored Per Game", allTimeTeamsStatSheet_[22][0])
        self.assertEquals("Opponent Points Scored", allTimeTeamsStatSheet_[23][0])
        self.assertEquals("Opponent Points Scored Per Game", allTimeTeamsStatSheet_[24][0])
        self.assertEquals("Scoring Share", allTimeTeamsStatSheet_[25][0])
        self.assertEquals("Opponent Scoring Share", allTimeTeamsStatSheet_[26][0])
        self.assertEquals("Max Scoring Share", allTimeTeamsStatSheet_[27][0])
        self.assertEquals("Min Scoring Share", allTimeTeamsStatSheet_[28][0])
        self.assertEquals("Max Score", allTimeTeamsStatSheet_[29][0])
        self.assertEquals("Min Score", allTimeTeamsStatSheet_[30][0])
        self.assertEquals("Scoring Standard Deviation", allTimeTeamsStatSheet_[31][0])
        self.assertEquals("Plus/Minus", allTimeTeamsStatSheet_[32][0])
        self.assertEquals("Team Score", allTimeTeamsStatSheet_[33][0])
        self.assertEquals("Team Success", allTimeTeamsStatSheet_[34][0])
        self.assertEquals("Team Luck", allTimeTeamsStatSheet_[35][0])



from __future__ import annotations

import copy
from dataclasses import dataclass
from typing import Any

from leeger.enum.MatchupType import MatchupType
from leeger.model.league import Year


@dataclass(kw_only=True)
class YearFilters:
    """
    Used to house filters that will be applied to a Year when navigating through it.
    """
    weekNumberStart: int  # week to start at (inclusive)
    weekNumberEnd: int  # week to end at (inclusive)
    includeMultiWeekMatchups: bool = True  # whether to include multi-week matchups or not
    onlyChampionship: bool = False  # only include championship weeks
    onlyPostSeason: bool = False  # only include playoff weeks
    onlyRegularSeason: bool = False  # only include regular season weeks

    @property
    def includeMatchupTypes(self) -> list[MatchupType]:
        if self.onlyChampionship:
            return [MatchupType.CHAMPIONSHIP]
        elif self.onlyPostSeason:
            return [MatchupType.PLAYOFF, MatchupType.CHAMPIONSHIP]
        elif self.onlyRegularSeason:
            return [MatchupType.REGULAR_SEASON]
        else:
            return [MatchupType.REGULAR_SEASON, MatchupType.PLAYOFF, MatchupType.CHAMPIONSHIP]

    @classmethod
    def preferredOrderWithTitle(cls, year: Year, **kwargs) -> list[tuple[str, Any]]:
        yearFilters = YearFilters.getForYear(year, **kwargs)
        return [
            ("Week Number Start", yearFilters.weekNumberStart),
            ("Week Number End", yearFilters.weekNumberEnd),
            ("Only Regular Season", yearFilters.onlyRegularSeason),
            ("Only Post Season", yearFilters.onlyPostSeason),
            ("Only Championship", yearFilters.onlyChampionship),
            ("Include Multi-Week Matchups", yearFilters.includeMultiWeekMatchups)
        ]

    @classmethod
    def getForYear(cls, year: Year, **kwargs) -> YearFilters:
        kwargsCopy = copy.deepcopy(kwargs)
        from leeger.exception import InvalidFilterException
        from leeger.util.GeneralUtil import GeneralUtil
        onlyChampionship = kwargsCopy.pop("onlyChampionship", False)
        onlyPostSeason = kwargsCopy.pop("onlyPostSeason", False)
        onlyRegularSeason = kwargsCopy.pop("onlyRegularSeason", False)
        weekNumberStart = kwargsCopy.pop("weekNumberStart", year.weeks[0].weekNumber)
        weekNumberEnd = kwargsCopy.pop("weekNumberEnd", year.weeks[-1].weekNumber)
        includeMultiWeekMatchups = kwargsCopy.pop("includeMultiWeekMatchups", True)
        kwargsCopy.pop("includeMatchupTypes", None)  # so we don't get an unused kwarg warning

        GeneralUtil.warnForUnusedKwargs(kwargsCopy)

        ####################
        # validate filters #
        ####################
        # type checks
        if not isinstance(onlyChampionship, bool):
            raise InvalidFilterException("'onlyChampionship' must be type 'bool'")
        if not isinstance(onlyPostSeason, bool):
            raise InvalidFilterException("'onlyPostSeason' must be type 'bool'")
        if not isinstance(onlyRegularSeason, bool):
            raise InvalidFilterException("'onlyRegularSeason' must be type 'bool'")
        if not isinstance(weekNumberStart, int):
            raise InvalidFilterException("'weekNumberStart' must be type 'int'")
        if not isinstance(weekNumberEnd, int):
            raise InvalidFilterException("'weekNumberEnd' must be type 'int'")
        if not isinstance(includeMultiWeekMatchups, bool):
            raise InvalidFilterException("'includeMultiWeekMatchups' must be type 'bool'")

        # logic checks
        if [onlyChampionship, onlyPostSeason, onlyRegularSeason].count(True) > 1:
            raise InvalidFilterException(
                "Only one of 'onlyChampionship', 'onlyPostSeason', 'onlyRegularSeason' can be True")
        if weekNumberStart < 1:
            raise InvalidFilterException("'weekNumberStart' cannot be less than 1.")
        if weekNumberEnd > len(year.weeks):
            raise InvalidFilterException("'weekNumberEnd' cannot be greater than the number of weeks in the year.")
        if weekNumberStart > weekNumberEnd:
            raise InvalidFilterException("'weekNumberStart' cannot be greater than 'weekNumberEnd'.")

        return YearFilters(weekNumberStart=weekNumberStart, weekNumberEnd=weekNumberEnd,
                           includeMultiWeekMatchups=includeMultiWeekMatchups,
                           onlyPostSeason=onlyPostSeason, onlyChampionship=onlyChampionship,
                           onlyRegularSeason=onlyRegularSeason)

    def asKwargs(self) -> dict[str, Any]:
        return {
            "weekNumberStart": self.weekNumberStart,
            "weekNumberEnd": self.weekNumberEnd,
            "includeMatchupTypes": self.includeMatchupTypes,
            "includeMultiWeekMatchups": self.includeMultiWeekMatchups,
            "onlyPostSeason": self.onlyPostSeason,
            "onlyChampionship": self.onlyChampionship,
            "onlyRegularSeason": self.onlyRegularSeason
        }
